package com.tzld.piaoquan.tencentad.service.impl;

import com.tzld.piaoquan.tencentad.common.base.CommonResponse;
import com.tzld.piaoquan.tencentad.common.base.ExceptionCodeEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdMuseTaskStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdVideoImageStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.SimilarityCodeEnum;
import com.tzld.piaoquan.tencentad.dao.mapper.AdVideoImageMapper;
import com.tzld.piaoquan.tencentad.model.po.AdMuseTask;
import com.tzld.piaoquan.tencentad.model.po.AdVideoImage;
import com.tzld.piaoquan.tencentad.model.po.AdVideoImageExample;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoImageParam;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoImageVo;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoParam;
import com.tzld.piaoquan.tencentad.service.AdMuseTaskService;
import com.tzld.piaoquan.tencentad.service.AdVideoImageService;
import com.tzld.piaoquan.tencentad.service.TencentService;
import com.tzld.piaoquan.tencentad.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static com.tzld.piaoquan.tencentad.common.base.ExceptionCodeEnum.PARAM_ERROR;

@Slf4j
@Service
public class AdVideoImageServiceImpl implements AdVideoImageService {

    @Autowired
    private AdVideoImageMapper adVideoImageMapper;

    @Autowired
    private TencentService tencentService;

    @Autowired
    private AdMuseTaskService adMuseTaskService;

    @Override
    public boolean addAdVideoImage(Long adVideoId, String url) {
        try {
            AdVideoImage adVideoImage = new AdVideoImage();
            adVideoImage.setAdVideoId(adVideoId);
            adVideoImage.setImageUrl(url);
            adVideoImageMapper.insertSelective(adVideoImage);
            return true;
        } catch (Exception e) {
            log.error("addAdVideoImage error {}", e.getMessage());
        }
        return false;
    }

    @Override
    public CommonResponse<List<AdVideoImageVo>> getAdVideoImageVoList(Long adVideoId) {
        AdVideoImageExample example = new AdVideoImageExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId).andIsDeleteEqualTo(0);
        List<AdVideoImage> adVideoImages = adVideoImageMapper.selectByExample(example);
        List<AdVideoImageVo> adVideoImageVos = new ArrayList<>();
        for (AdVideoImage adVideoImage : adVideoImages) {
            AdVideoImageVo adVideoImageVo = new AdVideoImageVo();
            BeanUtils.copyProperties(adVideoImage, adVideoImageVo);
            if (adVideoImage.getSimilarity() != null) {
                SimilarityCodeEnum similarityCodeEnum = SimilarityCodeEnum.getByCode(adVideoImage.getSimilarity());
                if (similarityCodeEnum != null) {
                    adVideoImageVo.setSimilarityName(similarityCodeEnum.getDesc());
                }
            }
            AdVideoImageStatusEnum adVideoImageStatusEnum = AdVideoImageStatusEnum.getByStatus(adVideoImage.getStatus());
            if (adVideoImageStatusEnum != null) {
                adVideoImageVo.setStatusName(adVideoImageStatusEnum.getDesc());
            }
            adVideoImageVo.setCreateTime(DateUtils.getDateString(adVideoImage.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adVideoImageVo.setUpdateTime(DateUtils.getDateString(adVideoImage.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adVideoImageVos.add(adVideoImageVo);
        }
        return CommonResponse.success(adVideoImageVos);
    }

    @Override
    public List<AdVideoImage> getAdVideoImageList(Long adVideoId, Integer status) {
        AdVideoImageExample example = new AdVideoImageExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId).andStatusEqualTo(status)
                .andIsDeleteEqualTo(0);
        return adVideoImageMapper.selectByExample(example);
    }

    public CommonResponse<Void> updateAdVideoImage(AdVideoImageParam adVideoImageParam) {
        if (adVideoImageParam == null || adVideoImageParam.getId() == null) {
            return CommonResponse.create(PARAM_ERROR.getCode(), PARAM_ERROR.getMsg());
        }
        AdVideoImage updateAdVideoImage = new AdVideoImage();
        updateAdVideoImage.setId(adVideoImageParam.getId());
        updateAdVideoImage.setStatus(adVideoImageParam.getStatus());
        updateAdVideoImage.setSimilarity(adVideoImageParam.getSimilarity());
        updateAdVideoImage.setDimensionSize(adVideoImageParam.getDimensionSize());
        updateAdVideoImage.setText2imgDesc(adVideoImageParam.getText2imgDesc());
        int update = adVideoImageMapper.updateByPrimaryKeySelective(updateAdVideoImage);
        if (update > 0) {
            return CommonResponse.success();
        } else {
            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "修改失败");
        }
    }

    //上传图片到腾讯
    public boolean uploadImageToTencent(String account, String accessToken, Long adVideoId) {
        List<AdVideoImage> videoImageList = getAdVideoImageList(adVideoId, AdVideoImageStatusEnum.VERIFY_SUCCESS.getStatus());
        if (CollectionUtils.isEmpty(videoImageList)) {
            return false;
        }
        boolean flag = false;
        for (AdVideoImage adVideoImage : videoImageList) {
            String imageId = tencentService.uploadImage(account, accessToken, adVideoImage.getImageUrl());
            if (StringUtils.isNotEmpty(imageId)) {
                adVideoImage.setTencentImageId(imageId);
                adVideoImage.setStatus(AdVideoImageStatusEnum.UPLOAD_SUCCESS.getStatus());
                adVideoImageMapper.updateByPrimaryKeySelective(adVideoImage);
                adMuseTaskService.addMuseTask(account, accessToken, adVideoImage);
                flag = true;
            } else {
                adVideoImage.setStatus(AdVideoImageStatusEnum.UPLOAD_FAIL.getStatus());
                adVideoImageMapper.updateByPrimaryKeySelective(adVideoImage);
            }
        }
        return flag;
    }

    public long selectCount(Long adVideoId, Integer status) {
        AdVideoImageExample example = new AdVideoImageExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId).andStatusEqualTo(status);
        return adVideoImageMapper.countByExample(example);
    }


}
